"""
Price formatting utilities with exchange-specific precision handling.
"""

import logging
import math
from typing import Dict, Optional, Any
from functools import lru_cache

logger = logging.getLogger(__name__)

def _normalize_token_case(token: str) -> str:
    """
    Normalize token case: if any characters are already uppercase, keep as-is.
    Otherwise, convert to uppercase. This handles mixed-case tokens like kPEPE, kBONK.
    """
    if any(c.isupper() for c in token):
        return token
    else:
        return token.upper()

class TokenDisplayFormatter:
    """Handles price and amount formatting with proper decimal precision from exchange data."""

    def __init__(self, trading_engine=None):
        """
        Initialize price formatter.

        Args:
            trading_engine: TradingEngine instance for accessing exchange data
        """
        self.trading_engine = trading_engine
        # Cache structure: {token: {'price_decimals': X, 'amount_decimals': Y}}
        self._precision_cache: Dict[str, Dict[str, int]] = {}
        self._markets_cache: Optional[Dict[str, Any]] = None

    def _load_markets_data(self) -> Dict[str, Any]:
        """Load markets data with caching."""
        if self._markets_cache is None and self.trading_engine:
            try:
                markets = self.trading_engine.client.get_markets()
                if markets:
                    self._markets_cache = markets
                    logger.info(f"📊 Loaded {len(markets)} markets for precision data")
                else:
                    logger.warning("⚠️ Could not load markets data from exchange")
                    self._markets_cache = {}
            except Exception as e:
                logger.error(f"❌ Error loading markets data: {e}")
                self._markets_cache = {}
        return self._markets_cache or {}

    def _fetch_and_cache_precisions(self, token: str) -> Optional[Dict[str, int]]:
        """
        Fetches price and amount precisions for a token from market data and caches them.
        Returns the cached dict {'price_decimals': X, 'amount_decimals': Y} or None if not found.
        """
        normalized_token = _normalize_token_case(token)
        if normalized_token in self._precision_cache:
            return self._precision_cache[normalized_token]

        markets = self._load_markets_data()
        if not markets:
            logger.debug(f"No markets data available for {normalized_token}, cannot fetch precisions.")
            return None

        possible_symbols = [
            f"{normalized_token}/USDC:USDC",
            f"{normalized_token}/USDC",
            # Add other common quote currencies or symbol formats if necessary
        ]
        # Handle k-tokens or other prefixed tokens if that's a pattern
        if normalized_token.startswith('K') and len(normalized_token) > 1 and not normalized_token[1].isnumeric(): # e.g. kPEPE but not KSM
             possible_symbols.append(f"{normalized_token}/USDC:USDC") # Already covered if normalized_token includes 'k'
             possible_symbols.append(f"{normalized_token.upper()}/USDC:USDC") # Try uppercase original if 'k' was part of normalization strategy

        market_info = None
        for symbol_variant in possible_symbols:
            if symbol_variant in markets:
                market_info = markets[symbol_variant]
                break
        
        if not market_info:
            # Try direct lookup for already fully qualified symbols (e.g. "PEPE/USDC:USDC")
            if token in markets:
                 market_info = markets[token]
            else:
                logger.warning(f"Market info not found for {normalized_token} or its variants.")
                return None

        precision_info = market_info.get('precision', {})
        price_precision_val = precision_info.get('price')
        amount_precision_val = precision_info.get('amount')

        price_decimals = self._get_default_price_decimals_for_token(normalized_token) # Default
        if price_precision_val and price_precision_val > 0:
            price_decimals = int(-math.log10(price_precision_val))

        # For amount, we typically need it from the exchange; default might be less useful
        # Defaulting to a common high precision if not found, but ideally exchange provides this.
        amount_decimals = 6 
        if amount_precision_val and amount_precision_val > 0:
            amount_decimals = int(-math.log10(amount_precision_val))
        
        self._precision_cache[normalized_token] = {
            'price_decimals': price_decimals,
            'amount_decimals': amount_decimals
        }
        logger.debug(f"📊 Cached precisions for {normalized_token}: price {price_decimals}, amount {amount_decimals}")
        return self._precision_cache[normalized_token]

    def get_token_price_decimal_places(self, token: str) -> int:
        """
        Get the number of decimal places for a token's price.
        """
        normalized_token = _normalize_token_case(token)
        precisions = self._precision_cache.get(normalized_token)
        if not precisions:
            precisions = self._fetch_and_cache_precisions(normalized_token)

        if precisions:
            return precisions['price_decimals']
        
        # Fallback to smart default if fetching failed completely
        return self._get_default_price_decimals_for_token(normalized_token)

    def get_token_amount_decimal_places(self, token: str) -> int:
        """
        Get the number of decimal places for a token's amount (quantity).
        """
        normalized_token = _normalize_token_case(token)
        precisions = self._precision_cache.get(normalized_token)
        if not precisions:
            precisions = self._fetch_and_cache_precisions(normalized_token)
            
        if precisions:
            return precisions['amount_decimals']
        
        # Fallback if fetching failed - consider a sensible default for amounts
        logger.warning(f"Amount precision not found for {normalized_token}, defaulting to 6.")
        return 6 # Default amount precision

    def _get_default_price_decimals_for_token(self, token: str) -> int: # Renamed from _get_default_decimals_for_token
        """Get smart default price decimal places based on token characteristics."""
        token_upper = token.upper() # Ensure consistent casing for checks

        if token_upper in ['BTC', 'ETH', 'BNB', 'SOL', 'ADA', 'DOT', 'AVAX', 'MATIC', 'LINK']:
            return 2
        if token_upper in ['DOGE', 'XRP', 'LTC', 'BCH', 'ETC', 'FIL', 'AAVE', 'UNI']:
            return 4
        if any(meme in token_upper for meme in ['PEPE', 'SHIB', 'FLOKI', 'BONK', 'WIF']): # DOGE already covered
            return 6
        return 4

    def format_price(self, price: float, token: str = None) -> str:
        """
        Format a price with appropriate decimal places.
        """
        try:
            decimal_places = 2 # Default if no token
            if token:
                decimal_places = self.get_token_price_decimal_places(token)
            else:
                # Smart default based on price magnitude if no token provided
                if price == 0: decimal_places = 2
                elif abs(price) >= 1000: decimal_places = 2
                elif abs(price) >= 1: decimal_places = 3
                elif abs(price) >= 0.01: decimal_places = 4
                elif abs(price) >= 0.0001: decimal_places = 6
                else: decimal_places = 8
            
            return f"{price:,.{decimal_places}f}"
        except Exception as e:
            logger.error(f"❌ Error formatting price {price} for {token}: {e}")
            return f"{price:,.2f}"

    def format_price_with_symbol(self, price: float, token: str = None) -> str:
        """
        Format a price with currency symbol and appropriate decimal places.
        """
        formatted_price = self.format_price(price, token)
        return f"${formatted_price}"

    def format_amount(self, amount: float, token: str) -> str:
        """
        Format an amount (quantity) with appropriate decimal places for the given token.
        """
        try:
            decimal_places = self.get_token_amount_decimal_places(token)
            return f"{amount:,.{decimal_places}f}"
        except Exception as e:
            logger.error(f"❌ Error formatting amount {amount} for {token}: {e}")
            # Fallback, ensuring a reasonable number of decimals for an amount
            return f"{amount:,.6f}"

    def clear_cache(self):
        """Clear the precision cache."""
        self._precision_cache.clear()
        self._markets_cache = None
        logger.info("🗑️ Cleared price formatter cache")

# Global formatter instance
_global_formatter: Optional[TokenDisplayFormatter] = None

def set_global_trading_engine(trading_engine):
    """Set the trading engine for the global formatter."""
    global _global_formatter
    if _global_formatter is None or _global_formatter.trading_engine is None:
        _global_formatter = TokenDisplayFormatter(trading_engine)
        logger.info("Global TokenDisplayFormatter initialized with trading engine.")
    elif trading_engine is not _global_formatter.trading_engine:
        _global_formatter.trading_engine = trading_engine
        _global_formatter.clear_cache() # Clear cache if engine changes
        logger.info("Global TokenDisplayFormatter updated with a new trading engine and cache cleared.")


def get_formatter() -> TokenDisplayFormatter:
    """Get the global formatter instance. Ensures trading_engine is set if possible."""
    global _global_formatter
    if _global_formatter is None:
        # Attempt to create a basic formatter.
        # It's better if set_global_trading_engine is called explicitly during app setup.
        logger.warning("TokenDisplayFormatter.get_formatter() called before trading_engine was set globally. Creating instance without it. Precision data may be limited to defaults.")
        _global_formatter = TokenDisplayFormatter()
    return _global_formatter

def format_price(price: float, token: str = None) -> str:
    """Convenience function to format price using global formatter."""
    return get_formatter().format_price(price, token)

def format_price_with_symbol(price: float, token: str = None) -> str:
    """Convenience function to format price with $ symbol using global formatter."""
    return get_formatter().format_price_with_symbol(price, token)

def format_amount(amount: float, token: str) -> str:
    """Convenience function to format amount using global formatter."""
    return get_formatter().format_amount(amount, token) 